@@ -33,7 +33,7 @@ class TwistedEngine(object):
 
     def connect(self):
         d = self._defer_to_thread(self._engine.connect)
-        d.addCallback(TwistedConnection)
+        d.addCallback(TwistedConnection, self)
         return d
 
     def execute(self, *args, **kwargs):
@@ -43,9 +43,49 @@ class TwistedEngine(object):
 
 
 class TwistedConnection(object):
-    def __init__(self, connection):
+    def __init__(self, connection, engine):
         super(TwistedConnection, self).__init__()
         self._connection = connection
+        self._engine = engine
+
+    def execute(self, *args, **kwargs):
+        d = self._engine._defer_to_thread(
+            self._connection.execute, *args, **kwargs)
+        d.addCallback(TwistedResultProxy, self._engine)
+        return d
+
+    def close(self, *args, **kwargs):
+        return self._engine._defer_to_thread(
+            self._connection.close, *args, **kwargs)
+
+    @property
+    def closed(self):
+        return self._connection.closed
+
+    def begin(self, *args, **kwargs):
+        d = self._engine._defer_to_thread(
+            self._connection.begin, *args, **kwargs)
+        d.addCallback(TwistedTransaction, self._engine)
+        return d
+
+    def in_transaction(self):
+        return self._connection.in_transaction()
+
+
+class TwistedTransaction(object):
+    def __init__(self, transaction, engine):
+        super(TwistedTransaction, self).__init__()
+        self._transaction = transaction
+        self._engine = engine
+
+    def commit(self):
+        return self._engine._defer_to_thread(self._transaction.commit)
+
+    def rollback(self):
+        return self._engine._defer_to_thread(self._transaction.rollback)
+
+    def close(self):
+        return self._engine._defer_to_thread(self._transaction.close)
 
 
 class TwistedResultProxy(object):
